from margument.log import throw


class SettingsProcessor:
    def __init__(self, settings, args_parser):
        self.settings = settings
        self.args_parser = args_parser

    def run(self):

        for setting in self.settings:
            setting.program_arguments.add_arguments(self.args_parser)

        user_arguments = None
        try:
            user_arguments = self.args_parser.parse_args()
        except AssertionError:
            throw("Required arguments were not specified.")

        parsed_settings = {}
        for setting in self.settings:
            setting.user_arguments = user_arguments

            setting.program_arguments.process_arguments(self.settings)

            setting.do_commands()

            setting.set()

            parsed_settings[setting.program_arguments.__class__.__name__] = setting.save()

        return parsed_settings



class Argument:
    def __init__(self,
                 name='',
                 abbreviation_name='',
                 full_name='',
                 value=None,
                 help_message='',
                 metavar='',
                 default='None',
                 to_save=False,
                 is_main=False,
                 command=None,
                 command_args=None):
        self.name = name
        self.abbreviation_name = abbreviation_name
        self.full_name = full_name
        self.value = value
        self.help_message = help_message
        self.metavar = metavar
        self.default = default
        self.to_save = to_save
        self.is_main = is_main
        self.command = command
        self.command_args = command_args

    @staticmethod
    def set_argument(argument):
        return Argument(
            name=argument.name,
            abbreviation_name=argument.abbreviation_name,
            full_name=argument.full_name,
            value=argument.value,
            help_message=argument.help_message,
            metavar=argument.metavar,
            default=argument.default,
            to_save=argument.to_save,
            is_main=argument.is_main,
            command=argument.command,
            command_args=argument.command_args)

    def set_value(self, value):
        self.value = value

    def set_command(self, command):
        self.command = command

    def set_command_args(self, command_args):
        self.command_args = command_args





class Options:

    def __init__(self, show_saved=False, save_different=False, save_main_arg_exists=False, custom_save=False):
        self.show_saved = show_saved
        self.save_different = save_different
        self.save_main_arg_exists = save_main_arg_exists
        self.custom_save = custom_save




from margument.argument import Argument
from margument.reflection import get_class_variables


class Arguments:
    def to_list(self, is_saved=None):
        if is_saved is None:
            is_saved = [True, False]

        class_variables = get_class_variables(self)

        arguments = []
        for var in class_variables:
            if not isinstance(var[1], Argument):
                continue

            arg = Argument.set_argument(var[1])

            if arg.to_save not in is_saved:
                continue

            arguments.append(arg)

        return arguments

    def from_list(self, arguments):
        class_variables = get_class_variables(self)

        for var in class_variables:
            if not isinstance(var[1], Argument):
                continue

            arg = Argument.set_argument(var[1])

            arg_from_list = None
            for argument in arguments:
                if argument.name != arg.name:
                    continue
                arg_from_list = argument

            if arg_from_list is None:
                continue

            setattr(self, arg.name, arg_from_list)

    def add_arguments(self, args_parser):
        raise NotImplementedError("This method needs to be implemented.")

    def process_arguments(self, settings):
        raise NotImplementedError("This method needs to be implemented.")